#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define BUFFER 100

unsigned int randaux()
{
	static long seed = 1;
	return (((seed = seed * 214013L + 2531011L) >> 16) & 0x7fff);
}

/* Implementação do procedimento MostraLamberta - imprime o tabuleiro */
void MostraLamberta(int arr[], int tamanho)
{

	for (int i = 0; i < tamanho; i++)
	{
		if (arr[i] == 0)
		{
			printf("%s", "O");
		}
		else
		{
			printf("%s", "X");
		}
	}
	printf("\n");
}

/* Implementação do procedimento ImprimeCabecalho - imprime o cabeçalho do tabuleiro*/
void ImprimeCabecalho(int tamanho)
{
	if (tamanho >= 100)
	{
		for (int i = 1; i < tamanho + 1; i++)
		{
			if (i % 100 != 0)
			{
				printf("%s", " ");
			}
			else
			{
				printf("%d", i / 100);
			}
		}
		printf("\n");
		for (int i = 1; i < tamanho + 1; i++)
		{
			if (i % 10 != 0)
			{
				printf("%s", " ");
			}
			else
			{

				printf("%d", (i % 100) / 10);
			}
		}
		printf("\n");
	}
	else
	{
		if (tamanho >= 10)
		{
			for (int i = 1; i < tamanho + 1; i++)
			{
				if (i % 10 != 0)
				{
					printf("%s", " ");
				}
				else
				{
					printf("%d", i / 10);
				}
			}
			printf("\n");
		}
	}
	for (int i = 1; i < tamanho + 1; i++)
	{
		printf("%d", i % 10);
	}
	printf("\n");
}

/* Implementação do procedimento GeraLamberta - gera um novo tabuleiro */
void GeraLamberta(int arr[], int tamanho)
{
	for (int i = 0; i < tamanho; i++)
	{
		int aux = randaux();
		if (aux % 2 == 0)
		{
			arr[i] = 0;
		}
		else
		{
			arr[i] = 1;
		}
	}
}

/* Implementação do procedimento JogadaValida - Vê se uma jogada é válida ou não */
int JogadaValida(int arr[], int casa, int elementos)
{
	/* verificar se nas posições existe pelo menos um um */
	for (int i = casa - 1; i < casa + elementos - 1; i++)
	{
		if (arr[i] == 1)
		{
			i = casa + elementos;
			return 1;
		}
	}
	return 0;
}

/* Implementação do procedimento EfetuaJogada  - efetua a jogada no tabuleiro */
void EfetuaJogada(int arr[], int casa, int elementos)
{
	for (int i = casa - 1; i < casa + elementos - 1; i++)
	{
		if (arr[i] == 0)
		{
			arr[i] = 1;
		}
		else
		{
			arr[i] = 0;
		}
	}
}

/* Implementação do procedimento ContaAlternancias  - efetua a contagem de alternancias entre a posicaoinicial e posicaofinal */
int ContaAlternancias(int arr[], int tamanho, int posicaoinicial, int tamanhosegmento)
{
	int alternancias = 0;
	int last_alternancias = 0;
	int posicaofinal = 0;
	if ((posicaoinicial + tamanhosegmento) > tamanho)
	{
		posicaofinal = tamanho;
	}
	else
	{
		posicaofinal = posicaoinicial + tamanhosegmento;
	}

	for (int i = posicaoinicial; i < posicaofinal - 1; i++)
	{
		if (arr[i + 1] != arr[i])
		{
			alternancias++;
		}
	}

	return alternancias;
}

 /* Implementação do procedimento MaximaAlternancias  - vê ual o máimo e alternancias e retorna a posicao */
int MaximaAlternancias(int arr[], int tamanho, int segmentos )
{

	int last_alternancias = 0;
	int casa=-1;
	int alternancias=0;
	for (int i = 0; i < tamanho - segmentos + 1; i++)
	{
		//MostraLambertaSegmento(arr, tamanho, i, i + segmentos);
		alternancias = ContaAlternancias(arr, tamanho, i, segmentos);
		//printf("   posicao inicial: %d nr arr %d ", i, segmentos);
		//printf("   Numero alternancias: %d\n", alternancias); 
		//printf("\n");
		if (alternancias > last_alternancias)
		{ 
			last_alternancias = alternancias;
			casa = i; 
		}
	}
	return casa;
}
/* Implementação do procedimento GeraJogada - decide qual a jogada a efetuar de acordo com as regras definidas */
/* recebe por referencia as variaveis casa, elementos e regra que altera para os valores pretendidos */
void GeraJogada(int arr[], int tamanho, int jogada, int *casa, int *elementos, int *regra)
{
	/* numero máximo de elementos a mexer = tamanho - jogada */

	/* 1. if tamanho - jogada <=1 então primeiro X */
	/* 2. Verifica se há um só bloco de X continuo e se sim é esse bloco */
	/* 3. if tamanho - jogada = 2 and numero de X par seleciona primeira segmento valido com dois elementos, se for impar seleciona primeiro segundo valido com um elemento */
	/* 4. Selecionar o maior segmento que tenha o maior número de alternâncias entre X e O. Existindo empate, selecionar o primeiro segmento.
	   Por exemplo, no segmento XXOX, existem duas alternâncias do X para o O e de O para X. */
	/* 5. Jogar o lance 1 1*/
	*regra = 0;
	if (tamanho - jogada <= 1) 
	{ 
		/* Regra 1 - tamanho - jogada <=1 então primeiro X*/
		/* encontrar o primeiro X */
		for (int i = 0; i < tamanho; i++)
		{
			if (arr[i] == 1)
			{
				*regra = 1;
				*casa = i + 1;
				*elementos = 1;
				i = tamanho;
			}
		}
		if (*regra == 0)
		{
			/* Não encontrou um X logo não pode ser a regra 1 */
			/* 5. Jogar o lance 1 1*/
			*regra = 5;
			*casa = 1;
			*elementos = 1;
		}
	}
	else
	{
		/* Regra 2 - Verifica se há um só bloco de X continuo e se sim é esse bloco*/
		int achou_x = 0;
		int quantos = 0;
		int blocos = 0;
		int anterior = 0;
		if (arr[0] == 1)
		{
			blocos = 1;
			anterior = 1;
			achou_x = 1;
			quantos = 1;
		}
		for (int i = 1; i < tamanho; i++)
		{
			if (arr[i] == 1)
			{
				if (anterior == 0)
				{
					blocos++;
					achou_x = i + 1;
				}
				quantos++;
				anterior = 1;
			}
			else
			{
				anterior = 0;
			}
		}
		if (blocos == 1)
		{
			// Há um só bloco de X
			*regra = 2;
			*casa = achou_x;
			if (tamanho - jogada < quantos)
			{
				*elementos = tamanho - jogada;
			}
			else
			{
				*elementos = quantos;
			}
		}
		else
		{
			/* Regra 3 - if tamanho - jogada = 2 and numero de X par seleciona primeira segmento valido com dois elementos (XX,OX ou XO),
			se for impar seleciona primeiro segundo valido com um elemento */
			if (tamanho - jogada == 2)
			{
				int numero_x = 0;
				int achou_x = 0;
				// Vamos contar o numero de elementos X
				for (int i = 0; i < tamanho; i++)
				{
					if (arr[i] == 1)
					{
						if (achou_x == 0)
						{
							// Guardamos a posição do primeiro X
							achou_x = i + 1;
						}
						numero_x = numero_x + 1;
					}
				}
				if (numero_x % 2 == 0)
				{
					/* Quantidade de X é par */
					*regra = 3;
					if (achou_x != 1)
					{
						*casa = achou_x - 1;
					}
					else
					{
						*casa = achou_x;
					}
					*elementos = 2;
				}
				else
				{
					/* Quantidade de X é impar */
					*regra = 3;
					*casa = achou_x;
					*elementos = 1;
				}
			}
			else
			{
				/* 4. Selecionar o maior segmento que tenha o maior número de alternâncias entre X e O.
				Existindo empate, selecionar o primeiro segmento.
				Por exemplo, no segmento XXOX, existem duas alternâncias do X para o O e de O para X. */
				
				int auxcasa=-1;
				int segmentos = 0;
				segmentos = tamanho - jogada;
				auxcasa = MaximaAlternancias(arr, tamanho, segmentos );
				 

				if (auxcasa !=-1)
				{
					*regra = 4;
					*casa = auxcasa + 1;
					*elementos = tamanho - jogada; 
				}
				 
				/* Tem que se verificar dentro do espaço tamanho-jogada....*/
				if (*regra == 0)
				{
					/* 5. Jogar o lance 1 1*/
					*regra = 5;
					*casa = 1;
					*elementos = 1;
				}
			}
		}
	}
}

void main()
{
	int i, valida, saltos, tamanho, jogada, casa, elementos, regra, tabuleiro[BUFFER];
	char term;
	/* valida a inserção de dados */
	if (scanf("%d %d%c", &tamanho, &saltos, &term) != 3 || term != '\n' || tamanho > BUFFER || tamanho < 1 || saltos < 0)
	{
		printf("valores errados insira dois inteiros positivos e o primeiro menor que %d\n", BUFFER);
	}
	else
	{

		// saltar os primeiros números aleatórios, para ter sequências distintas
		for (i = 0; i < saltos; i++)
		{
			randaux();
		}

		/* Chamar o procedimento para gerar um novo tabuleiro */
		GeraLamberta(tabuleiro, tamanho);
		/* Chamar o procedimento para imprimir o cabeçalho */
		ImprimeCabecalho(tamanho);
		/* Chamar o procedimento para mostrar o tabuleiro */
		MostraLamberta(tabuleiro, tamanho);

		jogada = 0;
		valida = 1;
		while (valida == 1)
		{
			casa = -1;
			elementos = -1;
			regra = -1;
			jogada++;
			// Geramos a jogada de acordo com as regras
			GeraJogada(tabuleiro, tamanho, jogada, &casa, &elementos, &regra);
			// Verifica se a jogada pretendida é válida
			valida = JogadaValida(tabuleiro, casa, elementos);

			printf("Jogada [%d]: -regra %d- %d %d\n", jogada, regra, casa, elementos);

			if (valida == 0)
			{
				// Jogada inválida
				if (jogada % 2 == 0)
				{
					printf("Jogada inválida, perde jogador 2.");
				}
				else
				{
					printf("Jogada inválida, perde jogador 1.");
				}
			}
			else
			{
				// Jogada válida

				/* Chamar o procedimento para efetuar a jogada no tabuleiro */
				EfetuaJogada(tabuleiro, casa, elementos);
				/* Chamar o procedimento para imprimir o cabeçalho */
				ImprimeCabecalho(tamanho);
				/* Chamar o procedimento para mostrar o tabuleiro */
				MostraLamberta(tabuleiro, tamanho);
			}
		}
	}
}
